#ifndef __CWAVEFUNCTION_WF_PPIPLUS_PHASESHIFT_CC__
#define __CWAVEFUNCTION_WF_PPIPLUS_PHASESHIFT_CC__
#include "wavefunction.h"

CWaveFunction_ppiplus_phaseshift::CWaveFunction_ppiplus_phaseshift(string  parsfilename) : CWaveFunction() {
  int iq,ichannel;
  double q;
  ParsInit(parsfilename);
	
  m1=MPROTON;
  m2=MPI;
	IDENTICAL=0;
  q1q2=1;
  if(COULOMB==0) q1q2=0;
  nchannels=3;
  ellmax=1;
  InitArrays();
  printf("Arrays Initialized\n");
	
  ell[0]=0;
  ell[1]=1;
  ell[2]=1;
	
  InitWaves();
  printf("Partial Waves Initialized\n");
	
  channelweight[0]=1.0;
  channelweight[1]=1.0; // J=1/2
  channelweight[2]=2.0; // J=3/2
	
  read_phaseshifts();
	
  for(ichannel=0;ichannel<nchannels;ichannel++){
    for(iq=0;iq<nqmax;iq++){
      q=qarray[iq];
      //printf("ichannel=%d, q=%g, delta=%g, ddeltadq=%g\n",
				//     ichannel,q,delta[ichannel][iq]*180.0/PI,
				//     ddeltadq[ichannel][iq]*180.0/PI);
      Wepsilon[ichannel][iq]=ddeltadq[ichannel][iq]
			-GetIW(ell[ichannel],epsilon,q,q1q2,eta[iq],delta[ichannel][iq])
			+GetIW(ell[ichannel],epsilon,q,q1q2,eta[iq],0.0);
      Wepsilon[ichannel][iq]=3.0*Wepsilon[ichannel][iq]
			/(4.0*PI*pow(epsilon,3));
    }
  }
  printf("Initialization finished\n");
}

double CWaveFunction_ppiplus_phaseshift::CalcPsiSquared(int iq,double r,double ctheta){
  double psisquared,x,dpsi2,q;
  double delta_s31,delta_p31,delta_p33;
  complex<double> psi,hstar,psi0;
  int ichannel;
  
  q=qarray[iq];
  if(iq>=nqmax){
    printf("iq too large!\n");
    exit(1);
  }
  psi0=planewave[iq]->planewave(r,ctheta);
	
  if(STRONG==1){
    if(r<epsilon){
      psisquared=real(psi0*conj(psi0));
      for(ichannel=0;ichannel<nchannels;ichannel++){
				dpsi2=channelweight[ichannel]*2.0*PI*Wepsilon[ichannel][iq]
				*pow(HBARC,3)/(q*q);
				psisquared+=dpsi2;
      }
    }
    else{
      x=q*r/HBARC;
      delta_s31=delta[0][iq];
      delta_p31=delta[1][iq];
      delta_p33=delta[2][iq];
			
      psi=psi0;
      // this refers to m_s=+1/2 s wave
      ichannel=0;
      hstar=partwave[ell[ichannel]][iq]->GetPhiIncoming(r)/x;
      psi+=0.5*hstar*(Misc::ceiphi(-2.0*delta_s31)-1.0);
			
      // m_s still equals 1/2, but now ell=1
      ichannel=1;
      hstar=partwave[ell[ichannel]][iq]->GetPhiIncoming(r)/x;
      psi+=0.5*hstar*((2.0/3.0)*Misc::ceiphi(-2.0*delta_p33)
				+(1.0/3.0)*Misc::ceiphi(-2.0*delta_p31)-1.0)*ci*(3.0)*ctheta; 
				psisquared=real(psi*conj(psi));
				
				//Now consider m_s=-1/2, m_ell=1, Note: doesn't interfere with m_s=+1/2
				ichannel=2;
				psi=0.5*hstar*(Misc::ceiphi(-2.0*delta_p33)-Misc::ceiphi(-2.0*delta_p31))
				*ci*(3.0)*sqrt(1.0-ctheta*ctheta)/3.0;
				psisquared+=real(psi*conj(psi));
				
    }
  }
  else psisquared=real(psi0*conj(psi0));
	psisquared*=RelativisticCorrection(r,iq);
  return psisquared;
	
}

void CWaveFunction_ppiplus_phaseshift::read_phaseshifts(){
	//#include "ppi_phaseshiftdat.cc"
	double delqdata=0.5;
	int nqdata=780;
	double data_delta[3][781]={{0,-4.77964e-08,-1.2604e-05,-8.24234e-05,-0.00021688,-0.000398094,-0.000610254,-0.000843016,-0.00108989,-0.00134676,-0.0016109,-0.00188048,-0.00215424,-0.00243125,-0.00271084,-0.00299253,-0.00327592,-0.00356072,-0.0038467,-0.00413368,-0.0044215,-0.00471005,-0.00499922,-0.00528893,-0.00557912,-0.00586973,-0.0061607,-0.00645199,-0.00674357,-0.00703541,-0.00732748,-0.00761975,-0.0079122,-0.00820482,-0.00849759,-0.00879049,-0.00908352,-0.00937665,-0.00966989,-0.00996322,-0.0102566,-0.0105501,-0.0108437,-0.0111373,-0.011431,-0.0117247,-0.0120185,-0.0123123,-0.0126061,-0.0129,-0.0131939,-0.0138719,-0.0141904,-0.0145104,-0.0148305,-0.0151508,-0.0154711,-0.0157916,-0.0161121,-0.0164326,-0.0167532,-0.0170739,-0.0173947,-0.0177155,-0.0180363,-0.0183572,-0.0186781,-0.0189991,-0.0193201,-0.0196412,-0.0199825,-0.0203338,-0.0206851,-0.0210365,-0.0213881,-0.0217397,-0.0220914,-0.0224431,-0.0227949,-0.0231468,-0.0234988,-0.0238508,-0.0242029,-0.024555,-0.0249072,-0.0252595,-0.0256118,-0.0259815,-0.0263653,-0.0267492,-0.0271332,-0.0275173,-0.0279014,-0.0282856,-0.0286699,-0.0290543,-0.0294387,-0.0298232,-0.0302077,-0.0305924,-0.030977,-0.0313617,-0.0317465,-0.0321313,-0.0325236,-0.0329216,-0.0333196,-0.0337177,-0.0341159,-0.0345141,-0.0349124,-0.0353107,-0.035709,-0.0361074,-0.0365059,-0.0369043,-0.0373029,-0.0377014,-0.0381,-0.0384987,-0.0388973,-0.0393278,-0.0397702,-0.0402127,-0.0406552,-0.0410978,-0.0415404,-0.0419831,-0.0424259,-0.0428687,-0.0433115,-0.0437544,-0.0441973,-0.0446403,-0.0450834,-0.0455264,-0.0459695,-0.0464141,-0.0468821,-0.0473502,-0.0478184,-0.0482866,-0.0487548,-0.0492231,-0.0496914,-0.0501598,-0.0506282,-0.0510967,-0.0515652,-0.0520337,-0.0525023,-0.0529709,-0.0534396,-0.0539083,-0.0543914,-0.0548859,-0.0553805,-0.0558751,-0.0563697,-0.0568644,-0.0573591,-0.0578539,-0.0583487,-0.0588435,-0.0593384,-0.0598333,-0.0603282,-0.0608232,-0.0613182,-0.0618133,-0.0623175,-0.0628497,-0.0633819,-0.0639141,-0.0644464,-0.0649787,-0.0655111,-0.0660435,-0.0665759,-0.0671084,-0.0676409,-0.0681734,-0.068706,-0.0692386,-0.0697713,-0.0703039,-0.0708388,-0.0713886,-0.0719384,-0.0724882,-0.073038,-0.0735879,-0.0741378,-0.0746878,-0.0752378,-0.0757878,-0.0763378,-0.0768879,-0.077438,-0.0779882,-0.0785383,-0.0790885,-0.0796467,-0.080236,-0.0808253,-0.0814146,-0.082004,-0.0825934,-0.0831829,-0.0837724,-0.0843619,-0.0849514,-0.085541,-0.0861306,-0.0867202,-0.0873099,-0.0878996,-0.0884893,-0.0890826,-0.0896798,-0.0902771,-0.0908743,-0.0914716,-0.092069,-0.0926663,-0.0932637,-0.0938611,-0.0944585,-0.095056,-0.0956534,-0.0962509,-0.0968485,-0.097446,-0.0980436,-0.0986695,-0.0992968,-0.0999242,-0.100552,-0.101179,-0.101807,-0.102434,-0.103062,-0.103689,-0.104317,-0.104944,-0.105572,-0.1062,-0.106827,-0.107455,-0.108095,-0.108742,-0.109389,-0.110036,-0.110683,-0.11133,-0.111977,-0.112624,-0.113271,-0.113919,-0.114566,-0.115213,-0.11586,-0.116508,-0.117155,-0.117812,-0.118479,-0.119146,-0.119813,-0.12048,-0.121148,-0.121815,-0.122482,-0.123149,-0.123817,-0.124484,-0.125151,-0.125818,-0.126486,-0.127153,-0.127832,-0.128519,-0.129207,-0.129894,-0.130582,-0.13127,-0.131957,-0.132645,-0.133333,-0.13402,-0.134708,-0.135396,-0.136084,-0.136772,-0.137459,-0.138145,-0.13883,-0.139515,-0.1402,-0.140885,-0.14157,-0.142255,-0.14294,-0.143625,-0.144311,-0.144996,-0.145681,-0.146366,-0.147051,-0.147742,-0.148448,-0.149154,-0.14986,-0.150566,-0.151271,-0.151977,-0.152683,-0.153389,-0.154095,-0.154802,-0.155508,-0.156214,-0.15692,-0.157626,-0.15834,-0.159055,-0.15977,-0.160485,-0.1612,-0.161916,-0.162631,-0.163346,-0.164061,-0.164777,-0.165492,-0.166207,-0.166922,-0.167638,-0.168351,-0.169063,-0.169775,-0.170487,-0.1712,-0.171912,-0.172624,-0.173337,-0.174049,-0.174761,-0.175473,-0.176186,-0.176898,-0.177611,-0.178337,-0.179071,-0.179805,-0.180538,-0.181272,-0.182006,-0.182739,-0.183473,-0.184207,-0.18494,-0.185674,-0.186408,-0.187142,-0.187876,-0.188607,-0.189337,-0.190067,-0.190798,-0.191528,-0.192259,-0.192989,-0.19372,-0.19445,-0.195181,-0.195911,-0.196642,-0.197373,-0.198102,-0.198829,-0.199556,-0.200283,-0.20101,-0.201737,-0.202464,-0.203191,-0.203918,-0.204645,-0.205372,-0.206099,-0.206826,-0.207553,-0.208286,-0.209021,-0.209757,-0.210493,-0.211229,-0.211965,-0.212701,-0.213437,-0.214173,-0.214908,-0.215644,-0.21638,-0.217116,-0.217855,-0.2186,-0.219345,-0.220089,-0.220834,-0.221579,-0.222324,-0.223069,-0.223814,-0.224559,-0.225303,-0.226048,-0.226793,-0.227538,-0.228278,-0.229019,-0.229759,-0.2305,-0.231241,-0.231981,-0.232722,-0.233462,-0.234203,-0.234944,-0.235685,-0.236425,-0.237166,-0.237902,-0.238638,-0.239374,-0.24011,-0.240846,-0.241582,-0.242318,-0.243054,-0.24379,-0.244527,-0.245263,-0.245999,-0.246737,-0.247482,-0.248226,-0.248971,-0.249716,-0.25046,-0.251205,-0.25195,-0.252694,-0.253439,-0.254184,-0.254929,-0.255673,-0.256416,-0.257155,-0.257895,-0.258634,-0.259374,-0.260114,-0.260853,-0.261593,-0.262333,-0.263073,-0.263812,-0.264552,-0.265292,-0.266027,-0.266761,-0.267495,-0.268229,-0.268964,-0.269698,-0.270432,-0.271166,-0.271901,-0.272635,-0.273369,-0.274104,-0.274842,-0.275584,-0.276327,-0.277069,-0.277812,-0.278554,-0.279297,-0.280039,-0.280782,-0.281524,-0.282267,-0.283009,-0.283751,-0.284488,-0.285224,-0.285961,-0.286698,-0.287434,-0.288171,-0.288907,-0.289644,-0.290381,-0.291117,-0.291854,-0.292591,-0.293322,-0.294052,-0.294782,-0.295513,-0.296243,-0.296973,-0.297704,-0.298434,-0.299165,-0.299895,-0.300626,-0.301356,-0.302081,-0.302804,-0.303528,-0.304252,-0.304976,-0.3057,-0.306424,-0.307147,-0.307871,-0.308595,-0.309319,-0.310043,-0.310774,-0.311506,-0.312237,-0.312969,-0.3137,-0.314432,-0.315163,-0.315895,-0.316626,-0.317358,-0.318089,-0.31882,-0.319544,-0.320269,-0.320993,-0.321717,-0.322442,-0.323166,-0.323891,-0.324615,-0.32534,-0.326064,-0.326789,-0.32751,-0.328227,-0.328944,-0.329661,-0.330378,-0.331095,-0.331812,-0.332529,-0.333246,-0.333963,-0.33468,-0.335397,-0.336107,-0.336816,-0.337526,-0.338235,-0.338944,-0.339654,-0.340363,-0.341072,-0.341782,-0.342491,-0.3432,-0.343906,-0.344607,-0.345308,-0.34601,-0.346711,-0.347412,-0.348113,-0.348815,-0.349516,-0.350217,-0.350918,-0.35162,-0.352328,-0.353036,-0.353745,-0.354453,-0.355161,-0.355869,-0.356577,-0.357285,-0.357993,-0.358702,-0.35941,-0.360098,-0.360782,-0.361466,-0.36215,-0.362834,-0.363518,-0.364202,-0.364886,-0.36557,-0.366255,-0.366939,-0.367628,-0.368318,-0.369009,-0.369699,-0.37039,-0.371081,-0.371771,-0.372462,-0.373153,-0.373844,-0.374534,-0.375208,-0.375873,-0.376539,-0.377205,-0.37787,-0.378536,-0.379202,-0.379867,-0.380533,-0.381199,-0.381865,-0.382535,-0.383207,-0.383879,-0.384551,-0.385222,-0.385894,-0.386566,-0.387238,-0.38791,-0.388582,-0.389254,-0.389901,-0.390547,-0.391193,-0.391839,-0.392485,-0.393131,-0.393777,-0.394423,-0.395069,-0.395715,-0.396362,-0.397014,-0.397666,-0.398318,-0.398969,-0.399621,-0.400273,-0.400925,-0.401577,-0.402229,-0.40288,-0.403517,-0.404142,-0.404767,-0.405391,-0.406016,-0.406641,-0.407266,-0.407891,-0.408516,-0.409141,-0.409766,-0.41038,-0.410994,-0.411608,-0.412222,-0.412835,-0.413449,-0.414063,-0.414677,-0.415291,-0.415905,-0.416513,-0.417115,-0.417718,-0.418321,-0.418923,-0.419526,-0.420129,-0.420731,-0.421334,-0.421937,-0.422533,-0.423107,-0.423681,-0.424255,-0.42483,-0.425404,-0.425978,-0.426553,-0.427127,-0.427701,-0.428275,-0.428838,-0.4294,-0.429962,-0.430525,-0.431087,-0.431649,-0.432211,-0.432773,-0.433336,-0.433898,-0.43445,-0.435,-0.43555,-0.4361,-0.43665,-0.437199,-0.437749,-0.438299,-0.438849,-0.439399,-0.439928,-0.440448,-0.440968,-0.441488,-0.442008,-0.442528,-0.443048,-0.443568,-0.444088,-0.444608,-0.445107,-0.445597,-0.446086,-0.446576,-0.447065,-0.447554,-0.448044,-0.448533,-0.449023,-0.449512,-0.449992,-0.450468,-0.450943,-0.451419,-0.451895,-0.452371,-0.452847,-0.453322,-0.453798,-0.454274,-0.454723,-0.455167,-0.455612,-0.456056,-0.4565,-0.456944,-0.457389,-0.457833,-0.458277,-0.458719,-0.459131,-0.459544,-0.459956,-0.460368,-0.46078,-0.461192,-0.461604,-0.462017,-0.462429,-0.462829,-0.463208,-0.463588,-0.463967,-0.471329},
		{0,-1.26537e-12,-5.75212e-10,-6.39403e-09,-2.65221e-08,-7.15686e-08,-1.52589e-07,-2.80793e-07,-4.67452e-07,-7.23865e-07,-1.06134e-06,-1.49121e-06,-2.02479e-06,-2.67341e-06,-3.44838e-06,-4.36105e-06,-5.42274e-06,-6.64476e-06,-8.03844e-06,-9.61512e-06,-1.13861e-05,-1.33627e-05,-1.55563e-05,-1.79781e-05,-2.06395e-05,-2.35518e-05,-2.67264e-05,-3.01744e-05,-3.39073e-05,-3.79363e-05,-4.22728e-05,-4.69281e-05,-5.19134e-05,-5.72402e-05,-6.29196e-05,-6.89629e-05,-7.53816e-05,-8.21869e-05,-8.939e-05,-9.70023e-05,-0.000105035,-0.0001135,-0.000122407,-0.000131769,-0.000141597,-0.000151901,-0.000162694,-0.000173987,-0.00018579,-0.000198115,-0.000210973,-0.000224377,-0.000238336,-0.000252862,-0.000267967,-0.000283661,-0.000299957,-0.000316865,-0.000334397,-0.000352563,-0.000371375,-0.000390845,-0.000410984,-0.000431802,-0.000453312,-0.000475524,-0.00049845,-0.000522101,-0.000546489,-0.000571623,-0.000597517,-0.00062418,-0.000651625,-0.000679863,-0.000708904,-0.00073876,-0.000769443,-0.000800963,-0.000833332,-0.000866561,-0.000900661,-0.000935644,-0.000971521,-0.0010083,-0.001046,-0.00108463,-0.00112419,-0.00116471,-0.00120618,-0.00124863,-0.00129206,-0.00133649,-0.00138192,-0.00142837,-0.00147585,-0.00152437,-0.00157394,-0.00162458,-0.00167628,-0.00172908,-0.00178296,-0.00183796,-0.00189407,-0.00195132,-0.0020097,-0.00206924,-0.00212994,-0.00219181,-0.00225487,-0.00231913,-0.0023846,-0.00245129,-0.0025192,-0.00258836,-0.00265877,-0.00273045,-0.0028034,-0.00287764,-0.00295318,-0.00303003,-0.0031082,-0.0031877,-0.00326854,-0.00335073,-0.0034343,-0.00351923,-0.00360556,-0.00369329,-0.00378242,-0.00387298,-0.00396497,-0.0040584,-0.00415329,-0.00424965,-0.00434748,-0.0044468,-0.00454762,-0.00464996,-0.00475381,-0.0048592,-0.00496613,-0.00507462,-0.00518468,-0.00529631,-0.00540954,-0.00552436,-0.0056408,-0.00575886,-0.00587855,-0.00599989,-0.00612289,-0.0053033,-0.00539585,-0.00548842,-0.00558734,-0.00569127,-0.00579523,-0.00589921,-0.00600322,-0.00610726,-0.00621132,-0.0063154,-0.00641951,-0.00652364,-0.00662779,-0.00673196,-0.00683616,-0.00694037,-0.00704461,-0.00714887,-0.00725862,-0.007385,-0.0075114,-0.00763784,-0.00776429,-0.00789077,-0.00801728,-0.00814381,-0.00827037,-0.00839694,-0.00852354,-0.00865016,-0.00877681,-0.00890347,-0.00903016,-0.00915686,-0.00928651,-0.00943596,-0.00958544,-0.00973495,-0.00988448,-0.010034,-0.0101836,-0.0103332,-0.0104828,-0.0106325,-0.0107822,-0.0109318,-0.0110816,-0.0112313,-0.011381,-0.0115308,-0.011681,-0.0118327,-0.0119844,-0.0121361,-0.0122879,-0.0124396,-0.0125914,-0.0127432,-0.012895,-0.0130468,-0.0131987,-0.0133506,-0.0135024,-0.0136544,-0.0138063,-0.0139582,-0.0141215,-0.0142972,-0.0144729,-0.0146486,-0.0148243,-0.0150001,-0.0151759,-0.0153517,-0.0155275,-0.0157033,-0.0158792,-0.016055,-0.0162309,-0.0164068,-0.0165827,-0.0167587,-0.0169578,-0.0171581,-0.0173585,-0.0175588,-0.0177592,-0.0179596,-0.01816,-0.0183605,-0.0185609,-0.0187614,-0.0189619,-0.0191624,-0.0193629,-0.0195635,-0.019764,-0.0199662,-0.0201694,-0.0203726,-0.0205758,-0.020779,-0.0209823,-0.0211855,-0.0213888,-0.0215921,-0.0217954,-0.0219987,-0.022202,-0.0224054,-0.0226087,-0.0228121,-0.0230279,-0.0232567,-0.0234855,-0.0237143,-0.0239431,-0.0241719,-0.0244008,-0.0246296,-0.0248585,-0.0250874,-0.0253163,-0.0255452,-0.0257742,-0.0260031,-0.0262321,-0.0264691,-0.0267125,-0.026956,-0.0271995,-0.027443,-0.0276866,-0.0279301,-0.0281737,-0.0284173,-0.0286609,-0.0289045,-0.0291481,-0.0293917,-0.0296354,-0.029879,-0.0301347,-0.0303932,-0.0306517,-0.0309102,-0.0311688,-0.0314273,-0.0316859,-0.0319445,-0.0322031,-0.0324617,-0.0327203,-0.032979,-0.0332376,-0.0334963,-0.0337558,-0.0340178,-0.0342798,-0.0345419,-0.0348039,-0.035066,-0.035328,-0.0355901,-0.0358522,-0.0361143,-0.0363764,-0.0366386,-0.0369007,-0.0371628,-0.037425,-0.0377112,-0.0380007,-0.0382903,-0.0385798,-0.0388694,-0.0391589,-0.0394485,-0.0397381,-0.0400277,-0.0403173,-0.0406069,-0.0408966,-0.0411862,-0.0414759,-0.0417765,-0.042082,-0.0423875,-0.042693,-0.0429985,-0.0433041,-0.0436096,-0.0439152,-0.0442208,-0.0445264,-0.044832,-0.0451376,-0.0454432,-0.0457489,-0.0460571,-0.0463666,-0.046676,-0.0469855,-0.0472951,-0.0476046,-0.0479141,-0.0482236,-0.0485332,-0.0488428,-0.0491523,-0.0494619,-0.0497715,-0.0500811,-0.0504044,-0.0507303,-0.0510562,-0.0513822,-0.0517081,-0.0520341,-0.0523601,-0.0526861,-0.0530121,-0.0533381,-0.0536641,-0.0539901,-0.0543162,-0.054645,-0.0549877,-0.0553304,-0.0556731,-0.0560158,-0.0563585,-0.0567012,-0.057044,-0.0573867,-0.0577295,-0.0580722,-0.058415,-0.0587578,-0.0591006,-0.0594462,-0.0597931,-0.0601401,-0.0604871,-0.060834,-0.061181,-0.061528,-0.061875,-0.0622221,-0.0625691,-0.0629161,-0.0632632,-0.0636103,-0.0639628,-0.064327,-0.0646911,-0.0650553,-0.0654195,-0.0657837,-0.0661479,-0.0665121,-0.0668764,-0.0672406,-0.0676049,-0.0679691,-0.0683334,-0.0686982,-0.0690668,-0.0694354,-0.069804,-0.0701726,-0.0705412,-0.0709098,-0.0712784,-0.071647,-0.0720157,-0.0723843,-0.072753,-0.0731217,-0.0734922,-0.0738784,-0.0742646,-0.0746509,-0.0750371,-0.0754234,-0.0758097,-0.0761959,-0.0765822,-0.0769685,-0.0773548,-0.0777411,-0.0781275,-0.0785148,-0.0789056,-0.0792963,-0.0796871,-0.0800779,-0.0804687,-0.0808595,-0.0812503,-0.0816412,-0.082032,-0.0824228,-0.0828137,-0.0832045,-0.0835976,-0.0839929,-0.0843881,-0.0847835,-0.0851788,-0.0855741,-0.0859694,-0.0863648,-0.0867601,-0.0871555,-0.0875508,-0.0879462,-0.0883416,-0.0887532,-0.0891667,-0.0895803,-0.0899938,-0.0904074,-0.0908209,-0.0912345,-0.0916481,-0.0920617,-0.0924753,-0.0928889,-0.0933025,-0.0937181,-0.0941363,-0.0945545,-0.0949726,-0.0953908,-0.095809,-0.0962272,-0.0966455,-0.0970637,-0.0974819,-0.0979002,-0.0983184,-0.0987373,-0.0991601,-0.0995829,-0.100006,-0.100428,-0.100851,-0.101274,-0.101697,-0.10212,-0.102543,-0.102966,-0.103388,-0.103811,-0.104238,-0.104666,-0.105093,-0.105521,-0.105948,-0.106376,-0.106803,-0.107231,-0.107658,-0.108085,-0.108513,-0.10894,-0.109372,-0.109804,-0.110236,-0.110668,-0.1111,-0.111532,-0.111964,-0.112396,-0.112828,-0.11326,-0.113692,-0.114124,-0.114561,-0.114997,-0.115434,-0.11587,-0.116307,-0.116744,-0.11718,-0.117617,-0.118054,-0.11849,-0.118927,-0.119364,-0.119805,-0.120246,-0.120688,-0.121129,-0.12157,-0.122011,-0.122452,-0.122893,-0.123335,-0.123776,-0.124217,-0.12466,-0.125106,-0.125552,-0.125997,-0.126443,-0.126889,-0.127334,-0.12778,-0.128226,-0.128671,-0.129117,-0.129563,-0.130013,-0.130463,-0.130913,-0.131363,-0.131813,-0.132264,-0.132714,-0.133164,-0.133614,-0.134065,-0.134515,-0.134967,-0.135422,-0.135876,-0.136331,-0.136786,-0.137241,-0.137695,-0.13815,-0.138605,-0.139059,-0.139514,-0.139969,-0.140429,-0.140888,-0.141347,-0.141806,-0.142265,-0.142724,-0.143184,-0.143643,-0.144102,-0.144561,-0.145021,-0.145484,-0.145947,-0.146411,-0.146874,-0.147338,-0.147802,-0.148265,-0.148729,-0.149193,-0.149657,-0.15012,-0.150587,-0.151055,-0.151523,-0.151991,-0.152459,-0.152928,-0.153396,-0.153864,-0.154332,-0.1548,-0.155268,-0.155739,-0.156212,-0.156684,-0.157157,-0.157629,-0.158102,-0.158575,-0.159047,-0.15952,-0.159992,-0.160465,-0.160941,-0.161418,-0.161895,-0.162372,-0.162848,-0.163325,-0.163802,-0.164279,-0.164756,-0.165233,-0.16571,-0.166191,-0.166673,-0.167154,-0.167635,-0.168117,-0.168598,-0.169079,-0.169561,-0.170042,-0.170523,-0.171006,-0.171491,-0.171977,-0.172463,-0.172949,-0.173434,-0.17392,-0.174406,-0.174891,-0.175377,-0.175863,-0.176361,-0.176867,-0.177373,-0.17788,-0.178386,-0.178892,-0.179399,-0.179905,-0.180411,-0.180918,-0.181424,-0.181935,-0.182446,-0.182957,-0.183468,-0.183978,-0.184489,-0.185,-0.185511,-0.186022,-0.186533,-0.187056,-0.187588,-0.18812,-0.188652,-0.189184,-0.189715,-0.190247,-0.190779,-0.191311,-0.191843,-0.19238,-0.192933,-0.193486,-0.194039,-0.194593,-0.195146,-0.195699,-0.196252,-0.196805,-0.197359,-0.197912,-0.19847,-0.199027,-0.199585,-0.200143,-0.200701,-0.201259,-0.201817,-0.202375,-0.202933,-0.203491,-0.204066,-0.204645,-0.205225,-0.205804,-0.206384,-0.206964,-0.207543,-0.208123,-0.208703,-0.209282,-0.209865,-0.21045,-0.211034,-0.211618,-0.212203,-0.212787,-0.213372,-0.213956,-0.214541,-0.215125,-0.215735,-0.216359,-0.216983,-0.217607,-0.218231,-0.218854,-0.219478,-0.220102,-0.220726,-0.22135,-0.221965,-0.222576,-0.223188,-0.223799,-0.224411,-0.225022,-0.225634,-0.226245,-0.226857,-0.227468,-0.228114,-0.228765,-0.229417,-0.230068,-0.23072,-0.231371,-0.232023,-0.232675,-0.233326,-0.233978,-0.234635,-0.235291,-0.235948,-0.236605,-0.237261,-0.237918,-0.238575,-0.239232,-0.239888,-0.240547,-0.241209,-0.241871,-0.242533,-0.247273},
	{0,6.32693e-12,2.8761e-09,3.19705e-08,1.32612e-07,3.57847e-07,7.62956e-07,1.40398e-06,2.33729e-06,3.61937e-06,5.30679e-06,7.45615e-06,1.01241e-05,1.33672e-05,1.72421e-05,2.18055e-05,2.7114e-05,3.32242e-05,4.01927e-05,4.80762e-05,5.69312e-05,6.68144e-05,7.77823e-05,8.98916e-05,0.000103199,0.000117761,0.000133633,0.000150874,0.000169539,0.000189684,0.000211367,0.000234643,0.00025957,0.000286204,0.000314602,0.000344819,0.000376912,0.000410939,0.000446955,0.000485017,0.000525182,0.000567505,0.000612044,0.000658854,0.000707993,0.000759516,0.000813481,0.000869943,0.000928959,0.000990586,0.00105488,0.0011219,0.00119169,0.00126432,0.00133985,0.00141832,0.0014998,0.00158434,0.001672,0.00176283,0.0018569,0.00195425,0.00205494,0.00215903,0.00226658,0.00237765,0.00249228,0.00261053,0.00273247,0.00285814,0.00298761,0.00312093,0.00325815,0.00339934,0.00354455,0.00369383,0.00384724,0.00400484,0.00416669,0.00433283,0.00450333,0.00467824,0.00485763,0.00504153,0.00523002,0.00542315,0.00562097,0.00582354,0.00603091,0.00624315,0.0064603,0.00668243,0.00690959,0.00714184,0.00737923,0.00762182,0.00786966,0.00812281,0.00838133,0.00864528,0.0089147,0.00918966,0.00947021,0.0097564,0.0100483,0.010346,0.0106494,0.0109588,0.0112741,0.0115953,0.0119226,0.012256,0.0125955,0.0129413,0.0132933,0.0136516,0.0140163,0.0143874,0.0147651,0.0151492,0.01554,0.0159374,0.0163415,0.0167524,0.0171701,0.0175946,0.0180261,0.0184646,0.0189102,0.0193628,0.0198226,0.0202896,0.0207638,0.0212454,0.0217344,0.0222308,0.0227346,0.023246,0.023765,0.0242917,0.0248261,0.0253682,0.0259181,0.0264759,0.0270417,0.0276154,0.0281971,0.028787,0.029385,0.0299912,0.0306056,0.0323667,0.0329729,0.0335793,0.0342758,0.0350435,0.0358114,0.0365795,0.0373479,0.0381164,0.0388851,0.0396541,0.0404232,0.0411925,0.041962,0.0427317,0.0435015,0.0442716,0.0450418,0.0458122,0.0466289,0.047586,0.0485433,0.0495009,0.0504586,0.0514166,0.0523748,0.0533332,0.0542918,0.0552505,0.0562095,0.0571687,0.058128,0.0590875,0.0600472,0.0610071,0.0619918,0.0631437,0.0642958,0.0654481,0.0666007,0.0677535,0.0689064,0.0700596,0.071213,0.0723666,0.0735203,0.0746743,0.0758285,0.0769828,0.0781373,0.079292,0.0804982,0.0819041,0.0833101,0.0847164,0.086123,0.0875297,0.0889367,0.0903439,0.0917513,0.093159,0.0945668,0.0959749,0.0973831,0.0987916,0.1002,0.101609,0.103147,0.104825,0.106503,0.108181,0.10986,0.111539,0.113218,0.114897,0.116577,0.118257,0.119937,0.121617,0.123298,0.124978,0.126659,0.12834,0.130327,0.132329,0.134331,0.136333,0.138336,0.140339,0.142342,0.144346,0.14635,0.148354,0.150358,0.152363,0.154368,0.156373,0.158378,0.160617,0.162996,0.165376,0.167757,0.170138,0.172519,0.1749,0.177282,0.179664,0.182046,0.184429,0.186812,0.189195,0.191579,0.193963,0.196563,0.199389,0.202214,0.205041,0.207867,0.210694,0.213522,0.21635,0.219178,0.222006,0.224835,0.227665,0.230495,0.233325,0.236155,0.239262,0.242593,0.245924,0.249255,0.252587,0.255919,0.259252,0.262585,0.265919,0.269254,0.272589,0.275924,0.27926,0.282596,0.285933,0.28976,0.293705,0.29765,0.301595,0.305542,0.309489,0.313436,0.317385,0.321334,0.325283,0.329234,0.333185,0.337136,0.341089,0.345216,0.349864,0.354513,0.359162,0.363813,0.368465,0.373117,0.377771,0.382425,0.38708,0.391736,0.396394,0.401052,0.405711,0.410371,0.415733,0.421193,0.426654,0.432116,0.43758,0.443045,0.448511,0.453978,0.459447,0.464917,0.470389,0.475862,0.481336,0.486811,0.492912,0.499296,0.505681,0.512069,0.518458,0.524849,0.531242,0.537636,0.544033,0.550431,0.556831,0.563234,0.569638,0.576043,0.583123,0.590525,0.59793,0.605338,0.612748,0.620161,0.627576,0.634994,0.642414,0.649837,0.657262,0.66469,0.67212,0.679553,0.687832,0.696278,0.704727,0.71318,0.721636,0.730095,0.738557,0.747023,0.755492,0.763964,0.772439,0.780918,0.789399,0.798043,0.807476,0.816914,0.826355,0.8358,0.845249,0.854701,0.864158,0.873619,0.883083,0.892551,0.902023,0.911498,0.920978,0.930956,0.94119,0.951428,0.961671,0.971917,0.982167,0.992422,1.00268,1.01294,1.02321,1.03348,1.04375,1.05403,1.06445,1.07517,1.08589,1.09661,1.10734,1.11807,1.1288,1.13954,1.15028,1.16103,1.17177,1.18253,1.19328,1.20405,1.21486,1.22569,1.23651,1.24734,1.25817,1.269,1.27984,1.29067,1.30151,1.31235,1.3232,1.33404,1.34485,1.35536,1.36587,1.37637,1.38688,1.3974,1.40791,1.41842,1.42893,1.43945,1.44997,1.46048,1.471,1.48137,1.49126,1.50114,1.51102,1.52091,1.53079,1.54067,1.55055,1.56044,1.57032,1.5802,1.59008,1.59996,1.60945,1.61852,1.62759,1.63666,1.64573,1.6548,1.66387,1.67294,1.682,1.69107,1.70013,1.70919,1.71825,1.72652,1.73469,1.74287,1.75104,1.75921,1.76738,1.77554,1.78371,1.79188,1.80004,1.8082,1.81636,1.82414,1.83143,1.83873,1.84602,1.85331,1.8606,1.86789,1.87518,1.88246,1.88975,1.89703,1.90431,1.9115,1.91798,1.92446,1.93095,1.93743,1.94391,1.95038,1.95686,1.96334,1.96981,1.97629,1.98276,1.98923,1.99505,2.00082,2.00659,2.01237,2.01814,2.02391,2.02968,2.03544,2.04121,2.04697,2.05274,2.0585,2.0637,2.06883,2.07396,2.07908,2.08421,2.08933,2.09445,2.09957,2.10469,2.10981,2.11493,2.12004,2.12467,2.12927,2.13388,2.13848,2.14308,2.14767,2.15227,2.15687,2.16147,2.16606,2.17066,2.17519,2.17933,2.18347,2.18762,2.19176,2.1959,2.20004,2.20418,2.20832,2.21246,2.2166,2.22073,2.2247,2.22845,2.2322,2.23596,2.23971,2.24346,2.24721,2.25096,2.25471,2.25846,2.26221,2.26596,2.26942,2.27283,2.27625,2.27967,2.28309,2.2865,2.28992,2.29333,2.29675,2.30016,2.30358,2.30687,2.31001,2.31315,2.31629,2.31942,2.32256,2.3257,2.32883,2.33197,2.33511,2.33824,2.34136,2.34428,2.34719,2.35011,2.35302,2.35594,2.35885,2.36177,2.36468,2.36759,2.37051,2.37342,2.37615,2.37884,2.38153,2.38422,2.3869,2.38959,2.39228,2.39497,2.39766,2.40035,2.40303,2.4056,2.40813,2.41065,2.41317,2.41569,2.41822,2.42074,2.42326,2.42578,2.4283,2.43082,2.43324,2.43561,2.43798,2.44034,2.44271,2.44508,2.44745,2.44982,2.45218,2.45455,2.45692,2.45919,2.46144,2.46368,2.46592,2.46817,2.47041,2.47265,2.4749,2.47714,2.47938,2.48162,2.48376,2.4859,2.48803,2.49016,2.4923,2.49443,2.49656,2.49869,2.50083,2.50296,2.50507,2.50711,2.50914,2.51118,2.51321,2.51525,2.51729,2.51932,2.52136,2.52339,2.52543,2.52741,2.52935,2.53128,2.53322,2.53516,2.5371,2.53903,2.54097,2.54291,2.54485,2.54678,2.54867,2.55056,2.55244,2.55433,2.55622,2.5581,2.55999,2.56188,2.56376,2.56565,2.56749,2.56929,2.57109,2.57289,2.5747,2.5765,2.5783,2.5801,2.5819,2.58371,2.5855,2.58726,2.58903,2.5908,2.59256,2.59433,2.5961,2.59786,2.59963,2.60139,2.60316,2.60487,2.60658,2.6083,2.61001,2.61172,2.61343,2.61515,2.61686,2.61857,2.62028,2.62196,2.62364,2.62531,2.62699,2.62866,2.63034,2.63201,2.63368,2.63536,2.63703,2.63868,2.64032,2.64195,2.64359,2.64522,2.64686,2.64849,2.65013,2.65177,2.6534,2.65502,2.65664,2.65825,2.65986,2.66148,2.66309,2.6647,2.66632,2.66793,2.66954,2.67113,2.6727,2.67427,2.67585,2.67742,2.679,2.68057,2.68214,2.68371,2.68529,2.68684,2.68839,2.68994,2.69149,2.69304,2.69459,2.69614,2.69769,2.69924,2.70079,2.70232,2.70384,2.70537,2.70689,2.70842,2.70995,2.71147,2.713,2.71453,2.71604,2.71754,2.71905,2.72055,2.71557}};
	double data_ddeltadq[3][781]={{0,-1.06513e-06,-7.04878e-05,-0.000209192,-0.000321949,-0.000397719,-0.000447705,-0.000481462,-0.000505001,-0.000521935,-0.000534465,-0.000543967,-0.00055133,-0.000557143,-0.000561807,-0.000565604,-0.000568734,-0.000571343,-0.000573541,-0.000575408,-0.000577008,-0.000578389,-0.000579588,-0.000580637,-0.000581559,-0.000582374,-0.000583097,-0.000583742,-0.00058432,-0.000584839,-0.000585307,-0.000585731,-0.000586116,-0.000586466,-0.000586785,-0.000587078,-0.000587346,-0.000587592,-0.000587819,-0.000588029,-0.000588223,-0.000588402,-0.000588569,-0.000588724,-0.000588868,-0.000589002,-0.000589127,-0.000589244,-0.000589353,-0.000589456,-0.000589552,-0.000638752,-0.000638984,-0.000642097,-0.000642315,-0.000642522,-0.000642719,-0.000642905,-0.000643082,-0.00064325,-0.000643411,-0.000643563,-0.000643709,-0.000643847,-0.00064398,-0.000644106,-0.000644227,-0.000644343,-0.000644454,-0.00064456,-0.000704871,-0.000705105,-0.00070533,-0.000705546,-0.000705753,-0.000705952,-0.000706144,-0.000706328,-0.000706506,-0.000706677,-0.000706842,-0.000707001,-0.000707154,-0.000707302,-0.000707445,-0.000707582,-0.000707716,-0.000770682,-0.0007709,-0.00077111,-0.000771313,-0.00077151,-0.000771701,-0.000771885,-0.000772064,-0.000772238,-0.000772406,-0.000772569,-0.000772727,-0.000772881,-0.00077303,-0.000773175,-0.000773316,-0.000773453,-0.000799745,-0.000799901,-0.000800053,-0.000800201,-0.000800345,-0.000800485,-0.000800622,-0.000800755,-0.000800884,-0.00080101,-0.000801133,-0.000801253,-0.00080137,-0.000801484,-0.000801595,-0.000801703,-0.000801809,-0.000889214,-0.000889381,-0.000889545,-0.000889705,-0.000889861,-0.000890014,-0.000890163,-0.000890309,-0.000890452,-0.000890591,-0.000890727,-0.000890861,-0.000890991,-0.000891118,-0.000891243,-0.000891366,-0.000941164,-0.000941311,-0.000941455,-0.000941596,-0.000941734,-0.00094187,-0.000942002,-0.000942132,-0.00094226,-0.000942385,-0.000942507,-0.000942628,-0.000942745,-0.000942861,-0.000942974,-0.000943086,-0.000943195,-0.000994796,-0.000994925,-0.000995053,-0.000995178,-0.000995301,-0.000995422,-0.00099554,-0.000995657,-0.000995771,-0.000995883,-0.000995994,-0.000996102,-0.000996209,-0.000996314,-0.000996417,-0.000996518,-0.00107071,-0.00107084,-0.00107096,-0.00107109,-0.00107121,-0.00107133,-0.00107145,-0.00107156,-0.00107168,-0.00107179,-0.0010719,-0.00107201,-0.00107211,-0.00107222,-0.00107232,-0.00107242,-0.00110654,-0.00110665,-0.00110676,-0.00110687,-0.00110698,-0.00110708,-0.00110718,-0.00110728,-0.00110738,-0.00110748,-0.00110758,-0.00110767,-0.00110777,-0.00110786,-0.00110795,-0.00110804,-0.00118619,-0.0011863,-0.00118641,-0.00118652,-0.00118663,-0.00118673,-0.00118683,-0.00118694,-0.00118704,-0.00118714,-0.00118723,-0.00118733,-0.00118743,-0.00118752,-0.00118761,-0.00118771,-0.00120267,-0.00120276,-0.00120286,-0.00120295,-0.00120304,-0.00120313,-0.00120321,-0.0012033,-0.00120339,-0.00120347,-0.00120355,-0.00120364,-0.00120372,-0.0012038,-0.00120388,-0.00120396,-0.00126351,-0.0012636,-0.00126369,-0.00126378,-0.00126387,-0.00126395,-0.00126404,-0.00126412,-0.00126421,-0.00126429,-0.00126437,-0.00126445,-0.00126453,-0.00126461,-0.00126469,-0.00130332,-0.00130341,-0.00130349,-0.00130358,-0.00130366,-0.00130374,-0.00130382,-0.0013039,-0.00130398,-0.00130406,-0.00130413,-0.00130421,-0.00130429,-0.00130436,-0.00130443,-0.00134393,-0.00134401,-0.00134409,-0.00134417,-0.00134425,-0.00134432,-0.0013444,-0.00134448,-0.00134455,-0.00134463,-0.0013447,-0.00134477,-0.00134484,-0.00134492,-0.00134499,-0.00138531,-0.00138538,-0.00138546,-0.00138554,-0.00138561,-0.00138568,-0.00138576,-0.00138583,-0.0013859,-0.00138597,-0.00138604,-0.00138611,-0.00138618,-0.00138625,-0.00138632,-0.00138066,-0.00138073,-0.00138079,-0.00138086,-0.00138092,-0.00138099,-0.00138105,-0.00138111,-0.00138118,-0.00138124,-0.0013813,-0.00138136,-0.00138143,-0.00138149,-0.00142274,-0.0014228,-0.00142287,-0.00142293,-0.001423,-0.00142306,-0.00142313,-0.00142319,-0.00142325,-0.00142331,-0.00142338,-0.00142344,-0.0014235,-0.00142356,-0.00142362,-0.00144157,-0.00144163,-0.00144169,-0.00144176,-0.00144182,-0.00144188,-0.00144193,-0.00144199,-0.00144205,-0.00144211,-0.00144217,-0.00144223,-0.00144228,-0.00144234,-0.00143602,-0.00143608,-0.00143613,-0.00143619,-0.00143624,-0.00143629,-0.00143635,-0.0014364,-0.00143645,-0.00143651,-0.00143656,-0.00143661,-0.00143666,-0.00143671,-0.00147916,-0.00147921,-0.00147927,-0.00147932,-0.00147938,-0.00147943,-0.00147949,-0.00147954,-0.00147959,-0.00147965,-0.0014797,-0.00147975,-0.00147981,-0.00147986,-0.00147298,-0.00147303,-0.00147308,-0.00147314,-0.00147319,-0.00147324,-0.00147329,-0.00147333,-0.00147338,-0.00147343,-0.00147348,-0.00147353,-0.00147358,-0.00146609,-0.00146614,-0.00146619,-0.00146623,-0.00146628,-0.00146632,-0.00146637,-0.00146642,-0.00146646,-0.00146651,-0.00146655,-0.0014666,-0.00146664,-0.00146669,-0.0014842,-0.00148425,-0.00148429,-0.00148434,-0.00148439,-0.00148443,-0.00148448,-0.00148452,-0.00148456,-0.00148461,-0.00148465,-0.0014847,-0.00148474,-0.00150225,-0.0015023,-0.00150234,-0.00150239,-0.00150243,-0.00150247,-0.00150252,-0.00150256,-0.00150261,-0.00150265,-0.00150269,-0.00150274,-0.00150278,-0.00149401,-0.00149405,-0.00149409,-0.00149414,-0.00149418,-0.00149422,-0.00149426,-0.0014943,-0.00149434,-0.00149438,-0.00149442,-0.00149446,-0.0014945,-0.00148508,-0.00148512,-0.00148516,-0.0014852,-0.00148524,-0.00148528,-0.00148532,-0.00148535,-0.00148539,-0.00148543,-0.00148547,-0.00148551,-0.00148554,-0.00150237,-0.00150241,-0.00150245,-0.00150249,-0.00150253,-0.00150257,-0.0015026,-0.00150264,-0.00150268,-0.00150272,-0.00150276,-0.00150279,-0.00150283,-0.00149241,-0.00149244,-0.00149248,-0.00149252,-0.00149255,-0.00149259,-0.00149263,-0.00149266,-0.0014927,-0.00149273,-0.00149277,-0.0014928,-0.00149284,-0.00148176,-0.00148179,-0.00148182,-0.00148186,-0.00148189,-0.00148193,-0.00148196,-0.00148199,-0.00148203,-0.00148206,-0.00148209,-0.00148213,-0.00149823,-0.00149826,-0.0014983,-0.00149833,-0.00149837,-0.0014984,-0.00149843,-0.00149847,-0.0014985,-0.00149854,-0.00149857,-0.0014986,-0.0014865,-0.00148654,-0.00148657,-0.0014866,-0.00148663,-0.00148666,-0.0014867,-0.00148673,-0.00148676,-0.00148679,-0.00148683,-0.00148686,-0.00148689,-0.00147412,-0.00147415,-0.00147419,-0.00147422,-0.00147425,-0.00147428,-0.00147431,-0.00147434,-0.00147437,-0.0014744,-0.00147443,-0.00147446,-0.00146103,-0.00146106,-0.00146109,-0.00146112,-0.00146115,-0.00146118,-0.0014612,-0.00146123,-0.00146126,-0.00146129,-0.00146132,-0.00146135,-0.00147635,-0.00147638,-0.00147641,-0.00147644,-0.00147647,-0.0014765,-0.00147653,-0.00147656,-0.00147659,-0.00147661,-0.00147664,-0.0014622,-0.00146223,-0.00146226,-0.00146228,-0.00146231,-0.00146234,-0.00146237,-0.00146239,-0.00146242,-0.00146245,-0.00146248,-0.0014625,-0.0014474,-0.00144743,-0.00144745,-0.00144748,-0.00144751,-0.00144753,-0.00144756,-0.00144758,-0.00144761,-0.00144764,-0.00144766,-0.00144769,-0.00143193,-0.00143195,-0.00143198,-0.001432,-0.00143203,-0.00143205,-0.00143208,-0.0014321,-0.00143213,-0.00143215,-0.00143218,-0.00141576,-0.00141579,-0.00141581,-0.00141583,-0.00141586,-0.00141588,-0.0014159,-0.00141593,-0.00141595,-0.00141597,-0.001416,-0.00142955,-0.00142958,-0.0014296,-0.00142963,-0.00142965,-0.00142967,-0.0014297,-0.00142972,-0.00142974,-0.00142977,-0.00142979,-0.00142981,-0.00138148,-0.0013815,-0.00138152,-0.00138154,-0.00138156,-0.00138158,-0.0013816,-0.00138162,-0.00138164,-0.00138166,-0.00138168,-0.00139455,-0.00139457,-0.00139459,-0.00139461,-0.00139464,-0.00139466,-0.00139468,-0.0013947,-0.00139472,-0.00139474,-0.00139476,-0.00134454,-0.00134455,-0.00134457,-0.00134459,-0.00134461,-0.00134462,-0.00134464,-0.00134466,-0.00134468,-0.00134469,-0.00134471,-0.0013569,-0.00135692,-0.00135694,-0.00135696,-0.00135698,-0.00135699,-0.00135701,-0.00135703,-0.00135705,-0.00135707,-0.00135708,-0.00130499,-0.001305,-0.00130502,-0.00130503,-0.00130505,-0.00130506,-0.00130508,-0.00130509,-0.00130511,-0.00130512,-0.00131664,-0.00131666,-0.00131667,-0.00131669,-0.0013167,-0.00131672,-0.00131673,-0.00131675,-0.00131677,-0.00131678,-0.0013168,-0.00126285,-0.00126286,-0.00126288,-0.00126289,-0.0012629,-0.00126291,-0.00126293,-0.00126294,-0.00126295,-0.00126296,-0.00124082,-0.00124083,-0.00124084,-0.00124085,-0.00124086,-0.00124087,-0.00124088,-0.00124089,-0.00124091,-0.00124092,-0.00124093,-0.00121816,-0.00121817,-0.00121818,-0.00121819,-0.0012182,-0.00121821,-0.00121822,-0.00121823,-0.00121824,-0.00121825,-0.00116128,-0.00116129,-0.00116129,-0.0011613,-0.00116131,-0.00116131,-0.00116132,-0.00116133,-0.00116134,-0.00116134,-0.00116135,-0.00113707,-0.00113707,-0.00113708,-0.00113708,-0.00113709,-0.0011371,-0.0011371,-0.00113711,-0.00113711,-0.00113712,-0.00111223,-0.00111223,-0.00111224,-0.00111224,-0.00111225,-0.00111225,-0.00111226,-0.00111226,-0.00111227,-0.00111227,-0.00105233,-0.00105233,-0.00105233,-0.00105233,-0.00105234,-0.00105234,-0.00105234,-0.00105234,-0.00105234,-0.00105234,-0.000991236,-0.000991235,-0.000991234,-0.000991233,-0.000991232,-0.000991231,-0.00099123,-0.000991229,-0.000991228,-0.000991228,-0.000963978,-0.000963976,-0.000963974,-0.000963972,-0.00096397,-0.000963969,-0.000963967,-0.000963965,-0.000963963,-0.000963961,-0.00090082,-0.000900815,-0.000900811,-0.000900806,-0.000900802,-0.000900798,-0.000900793,-0.000900789,-0.000900785,-0.0008365,-0.000836493,-0.000836487,-0.00083648,-0.000836473,-0.000836466,-0.000836459,-0.000836453,-0.000836446,-0.000836439,-0.000771017,-0.000771008,-0.000770998,-0.000770989,-0.000760909},
		{0,-2.94196e-11,-3.86092e-09,-2.25458e-08,-6.15358e-08,-1.22357e-07,-2.05496e-07,-3.11139e-07,-4.39366e-07,-5.90218e-07,-7.63715e-07,-9.59869e-07,-1.17869e-06,-1.42017e-06,-1.68432e-06,-1.97115e-06,-2.28064e-06,-2.6128e-06,-2.96763e-06,-3.34513e-06,-3.74529e-06,-4.16812e-06,-4.61362e-06,-5.08177e-06,-5.5726e-06,-6.08608e-06,-6.62222e-06,-7.18102e-06,-7.76248e-06,-8.3666e-06,-8.99337e-06,-9.64279e-06,-1.03149e-05,-1.10096e-05,-1.1727e-05,-1.2467e-05,-1.32297e-05,-1.4015e-05,-1.48229e-05,-1.56535e-05,-1.65068e-05,-1.73827e-05,-1.82812e-05,-1.92023e-05,-2.01461e-05,-2.11126e-05,-2.21016e-05,-2.31133e-05,-2.41477e-05,-2.52046e-05,-2.62842e-05,-2.73864e-05,-2.85112e-05,-2.96587e-05,-3.08288e-05,-3.20215e-05,-3.32368e-05,-3.44748e-05,-3.57353e-05,-3.70185e-05,-3.83243e-05,-3.96527e-05,-4.10038e-05,-4.23774e-05,-4.37737e-05,-4.51925e-05,-4.6634e-05,-4.80981e-05,-4.95847e-05,-5.1094e-05,-5.26259e-05,-5.41804e-05,-5.57575e-05,-5.73572e-05,-5.89795e-05,-6.06244e-05,-6.22918e-05,-6.39819e-05,-6.56946e-05,-6.74299e-05,-6.91877e-05,-7.09682e-05,-7.27712e-05,-7.45968e-05,-7.64451e-05,-7.83159e-05,-8.02092e-05,-8.21252e-05,-8.40638e-05,-8.60249e-05,-8.80086e-05,-9.00149e-05,-9.20438e-05,-9.40952e-05,-9.61692e-05,-9.82658e-05,-0.000100385,-0.000102527,-0.000104691,-0.000106878,-0.000109087,-0.000111319,-0.000113574,-0.000115851,-0.000118151,-0.000120473,-0.000122818,-0.000125185,-0.000127575,-0.000129988,-0.000132423,-0.000134881,-0.000137361,-0.000139864,-0.000142389,-0.000144937,-0.000147507,-0.0001501,-0.000152716,-0.000155354,-0.000158015,-0.000160698,-0.000163403,-0.000166132,-0.000168882,-0.000171656,-0.000174452,-0.00017727,-0.000180111,-0.000182975,-0.000185861,-0.000188769,-0.0001917,-0.000194654,-0.00019763,-0.000200629,-0.00020365,-0.000206694,-0.00020976,-0.000212849,-0.00021596,-0.000219094,-0.00022225,-0.000225429,-0.000228631,-0.000231855,-0.000235101,-0.00023837,-0.000241662,-0.000244975,-0.000248312,-0.00018564,-0.000185696,-0.00018575,-0.000208432,-0.000208495,-0.000208557,-0.000208618,-0.000208678,-0.000208737,-0.000208795,-0.000208851,-0.000208907,-0.000208962,-0.000209015,-0.000209068,-0.00020912,-0.000209171,-0.000209221,-0.00020927,-0.00025348,-0.000253545,-0.000253608,-0.000253671,-0.000253732,-0.000253793,-0.000253852,-0.000253911,-0.000253968,-0.000254025,-0.000254081,-0.000254135,-0.000254189,-0.000254242,-0.000254295,-0.000254346,-0.000299808,-0.000299872,-0.000299936,-0.000299999,-0.00030006,-0.000300121,-0.000300181,-0.00030024,-0.000300298,-0.000300355,-0.000300412,-0.000300467,-0.000300522,-0.000300576,-0.000300629,-0.000300682,-0.000304471,-0.000304523,-0.000304575,-0.000304625,-0.000304675,-0.000304724,-0.000304773,-0.00030482,-0.000304867,-0.000304914,-0.00030496,-0.000305005,-0.00030505,-0.000305094,-0.000305137,-0.00030518,-0.000352652,-0.000352705,-0.000352757,-0.000352809,-0.00035286,-0.000352911,-0.000352961,-0.00035301,-0.000353058,-0.000353106,-0.000353154,-0.000353201,-0.000353247,-0.000353293,-0.000353338,-0.000353382,-0.000402146,-0.0004022,-0.000402253,-0.000402306,-0.000402357,-0.000402409,-0.000402459,-0.000402509,-0.000402559,-0.000402607,-0.000402656,-0.000402703,-0.000402751,-0.000402797,-0.000402843,-0.00040807,-0.000408116,-0.000408161,-0.000408206,-0.000408251,-0.000408295,-0.000408338,-0.000408381,-0.000408424,-0.000408466,-0.000408508,-0.000408549,-0.00040859,-0.00040863,-0.00040867,-0.000459432,-0.00045948,-0.000459527,-0.000459574,-0.00045962,-0.000459665,-0.000459711,-0.000459755,-0.000459799,-0.000459843,-0.000459887,-0.00045993,-0.000459972,-0.000460014,-0.000460056,-0.000489065,-0.00048911,-0.000489155,-0.000489199,-0.000489243,-0.000489286,-0.00048933,-0.000489372,-0.000489414,-0.000489456,-0.000489498,-0.000489539,-0.000489579,-0.00048962,-0.00048966,-0.000519342,-0.000519385,-0.000519428,-0.000519471,-0.000519513,-0.000519554,-0.000519596,-0.000519637,-0.000519677,-0.000519717,-0.000519757,-0.000519797,-0.000519836,-0.000519875,-0.00052654,-0.000526579,-0.000526617,-0.000526656,-0.000526694,-0.000526731,-0.000526768,-0.000526805,-0.000526842,-0.000526878,-0.000526914,-0.00052695,-0.000526986,-0.000527021,-0.000527055,-0.000581755,-0.000581796,-0.000581836,-0.000581876,-0.000581916,-0.000581955,-0.000581994,-0.000582033,-0.000582072,-0.00058211,-0.000582148,-0.000582185,-0.000582222,-0.000582259,-0.000613914,-0.000613954,-0.000613994,-0.000614033,-0.000614072,-0.00061411,-0.000614149,-0.000614187,-0.000614224,-0.000614262,-0.000614299,-0.000614336,-0.000614372,-0.000614409,-0.000622072,-0.000622109,-0.000622145,-0.000622181,-0.000622217,-0.000622252,-0.000622288,-0.000622322,-0.000622357,-0.000622392,-0.000622426,-0.00062246,-0.000622493,-0.000622527,-0.000655154,-0.00065519,-0.000655226,-0.000655262,-0.000655297,-0.000655332,-0.000655367,-0.000655402,-0.000655436,-0.00065547,-0.000655504,-0.000655538,-0.000655571,-0.000688826,-0.000688862,-0.000688898,-0.000688934,-0.000688969,-0.000689004,-0.000689039,-0.000689074,-0.000689108,-0.000689142,-0.000689176,-0.00068921,-0.000689243,-0.000689277,-0.000697572,-0.000697606,-0.000697639,-0.000697672,-0.000697705,-0.000697738,-0.00069777,-0.000697803,-0.000697835,-0.000697867,-0.000697899,-0.00069793,-0.000697961,-0.000732149,-0.000732183,-0.000732217,-0.00073225,-0.000732283,-0.000732316,-0.000732349,-0.000732382,-0.000732414,-0.000732446,-0.000732478,-0.00073251,-0.000732541,-0.000741128,-0.00074116,-0.000741192,-0.000741224,-0.000741255,-0.000741286,-0.000741317,-0.000741348,-0.000741378,-0.000741409,-0.000741439,-0.000741469,-0.000741499,-0.000776598,-0.00077663,-0.000776662,-0.000776694,-0.000776726,-0.000776758,-0.000776789,-0.00077682,-0.000776851,-0.000776882,-0.000776913,-0.000776943,-0.000776973,-0.000785836,-0.000785866,-0.000785897,-0.000785927,-0.000785957,-0.000785987,-0.000786017,-0.000786047,-0.000786077,-0.000786106,-0.000786135,-0.000786164,-0.000786193,-0.000795035,-0.000795064,-0.000795093,-0.000795122,-0.000795151,-0.00079518,-0.000795208,-0.000795237,-0.000795265,-0.000795293,-0.000795321,-0.000795349,-0.000795376,-0.000831658,-0.000831687,-0.000831717,-0.000831747,-0.000831776,-0.000831805,-0.000831834,-0.000831863,-0.000831892,-0.000831921,-0.000831949,-0.000831978,-0.000841076,-0.000841105,-0.000841133,-0.000841162,-0.00084119,-0.000841218,-0.000841247,-0.000841274,-0.000841302,-0.00084133,-0.000841358,-0.000841385,-0.000850456,-0.000850484,-0.000850511,-0.000850539,-0.000850566,-0.000850593,-0.00085062,-0.000850647,-0.000850674,-0.000850701,-0.000850728,-0.000850754,-0.000850781,-0.000859823,-0.00085985,-0.000859876,-0.000859903,-0.000859929,-0.000859956,-0.000859982,-0.000860008,-0.000860034,-0.00086006,-0.000860085,-0.000860111,-0.000869123,-0.000869149,-0.000869175,-0.000869201,-0.000869227,-0.000869252,-0.000869278,-0.000869303,-0.000869328,-0.000869353,-0.000869378,-0.000869403,-0.000878384,-0.00087841,-0.000878435,-0.00087846,-0.000878485,-0.00087851,-0.000878534,-0.000878559,-0.000878583,-0.000878608,-0.000878632,-0.000887581,-0.000887606,-0.000887631,-0.000887655,-0.000887679,-0.000887704,-0.000887728,-0.000887752,-0.000887776,-0.0008878,-0.000887824,-0.000887848,-0.000896764,-0.000896788,-0.000896812,-0.000896836,-0.00089686,-0.000896884,-0.000896907,-0.000896931,-0.000896954,-0.000896978,-0.000897001,-0.000897024,-0.000905908,-0.000905932,-0.000905955,-0.000905978,-0.000906002,-0.000906025,-0.000906048,-0.000906071,-0.000906094,-0.000906117,-0.000906139,-0.00091499,-0.000915013,-0.000915036,-0.000915059,-0.000915081,-0.000915104,-0.000915127,-0.00091515,-0.000915172,-0.000915195,-0.000915217,-0.000924033,-0.000924056,-0.000924078,-0.000924101,-0.000924123,-0.000924146,-0.000924168,-0.000924191,-0.000924213,-0.000924235,-0.000924257,-0.000924279,-0.000933061,-0.000933083,-0.000933106,-0.000933128,-0.00093315,-0.000933172,-0.000933194,-0.000933216,-0.000933237,-0.000933259,-0.000933281,-0.000942029,-0.000942051,-0.000942073,-0.000942094,-0.000942116,-0.000942138,-0.00094216,-0.000942181,-0.000942203,-0.000942224,-0.000942246,-0.000950959,-0.00095098,-0.000951002,-0.000951024,-0.000951045,-0.000951067,-0.000951088,-0.000951109,-0.000951131,-0.000951152,-0.000951173,-0.000959852,-0.000959873,-0.000959895,-0.000959916,-0.000959937,-0.000959958,-0.000959979,-0.000960001,-0.000960022,-0.000960043,-0.000960064,-0.000968708,-0.000968729,-0.00096875,-0.000968771,-0.000968792,-0.000968813,-0.000968834,-0.000968855,-0.000968876,-0.000968897,-0.000977506,-0.000977527,-0.000977548,-0.000977569,-0.00097759,-0.000977611,-0.000977631,-0.000977652,-0.000977673,-0.000977693,-0.000977714,-0.00101885,-0.00101887,-0.00101889,-0.00101892,-0.00101894,-0.00101896,-0.00101898,-0.001019,-0.00101903,-0.00101905,-0.00102787,-0.0010279,-0.00102792,-0.00102794,-0.00102796,-0.00102799,-0.00102801,-0.00102803,-0.00102805,-0.00102807,-0.00102809,-0.00107002,-0.00107005,-0.00107007,-0.00107009,-0.00107012,-0.00107014,-0.00107016,-0.00107019,-0.00107021,-0.00107023,-0.0011127,-0.00111272,-0.00111275,-0.00111277,-0.0011128,-0.00111282,-0.00111285,-0.00111287,-0.0011129,-0.00111292,-0.00111294,-0.00112223,-0.00112225,-0.00112228,-0.0011223,-0.00112233,-0.00112235,-0.00112238,-0.0011224,-0.00112243,-0.00112245,-0.00116569,-0.00116572,-0.00116575,-0.00116577,-0.0011658,-0.00116583,-0.00116585,-0.00116588,-0.0011659,-0.00116593,-0.00117542,-0.00117545,-0.00117547,-0.0011755,-0.00117552,-0.00117555,-0.00117558,-0.0011756,-0.00117563,-0.00117566,-0.00125423,-0.00125426,-0.00125429,-0.00125432,-0.00125435,-0.00125438,-0.00125441,-0.00125444,-0.00125447,-0.0012545,-0.00122962,-0.00122965,-0.00122968,-0.0012297,-0.00122973,-0.00122976,-0.00122979,-0.00122981,-0.00122984,-0.00122987,-0.00130977,-0.0013098,-0.00130983,-0.00130987,-0.0013099,-0.00130993,-0.00130996,-0.00130999,-0.00131002,-0.00132019,-0.00132022,-0.00132025,-0.00132028,-0.00132031,-0.00132034,-0.00132037,-0.0013204,-0.00132043,-0.00132047,-0.00133059,-0.00133062,-0.00133065,-0.00133068,-0.00134065},
	{0,1.471e-10,1.93048e-08,1.12731e-07,3.07683e-07,6.11793e-07,1.02749e-06,1.55571e-06,2.19686e-06,2.95112e-06,3.81862e-06,4.7994e-06,5.8935e-06,7.10094e-06,8.42172e-06,9.85584e-06,1.14033e-05,1.30641e-05,1.48383e-05,1.67258e-05,1.87267e-05,2.08409e-05,2.30684e-05,2.54092e-05,2.78633e-05,3.04308e-05,3.31115e-05,3.59055e-05,3.88129e-05,4.18335e-05,4.49674e-05,4.82145e-05,5.15749e-05,5.50486e-05,5.86355e-05,6.23357e-05,6.61491e-05,7.00757e-05,7.41155e-05,7.82686e-05,8.25348e-05,8.69143e-05,9.1407e-05,9.60128e-05,0.000100732,0.000105564,0.000110509,0.000115568,0.00012074,0.000126024,0.000131422,0.000136934,0.000142558,0.000148295,0.000154146,0.000160109,0.000166186,0.000172376,0.000178678,0.000185094,0.000191623,0.000198265,0.00020502,0.000211889,0.00021887,0.000225964,0.000233171,0.000240492,0.000247925,0.000255471,0.00026313,0.000270903,0.000278788,0.000286786,0.000294897,0.000303121,0.000311459,0.000319909,0.000328471,0.000337147,0.000345936,0.000354838,0.000363852,0.00037298,0.00038222,0.000391573,0.000401039,0.000410618,0.000420309,0.000430113,0.000440031,0.000450061,0.000460203,0.000470459,0.000480827,0.000491308,0.000501901,0.000512607,0.000523426,0.000534358,0.000545402,0.000556559,0.000567828,0.00057921,0.000590704,0.000602311,0.000614031,0.000625863,0.000637807,0.000649864,0.000662033,0.000674315,0.000686708,0.000699215,0.000711833,0.000724564,0.000737406,0.000750361,0.000763429,0.000776608,0.000789899,0.000803302,0.000816817,0.000830445,0.000844183,0.000858034,0.000871997,0.000886071,0.000900257,0.000914555,0.000928964,0.000943484,0.000958116,0.00097286,0.000987714,0.00100268,0.00101776,0.00103295,0.00104824,0.00106365,0.00107918,0.00109481,0.00111055,0.0011264,0.00114236,0.00115844,0.00117462,0.00119092,0.00120732,0.00122383,0.00124046,0.00121575,0.00121613,0.00121651,0.00153878,0.0015393,0.00153981,0.0015403,0.00154079,0.00154127,0.00154174,0.00154221,0.00154266,0.00154311,0.00154355,0.00154398,0.0015444,0.00154482,0.00154523,0.00154563,0.00191877,0.00191931,0.00191984,0.00192036,0.00192087,0.00192138,0.00192188,0.00192236,0.00192285,0.00192332,0.00192379,0.00192424,0.0019247,0.00192514,0.00192558,0.00192601,0.00230971,0.00231026,0.0023108,0.00231134,0.00231187,0.00231239,0.0023129,0.0023134,0.0023139,0.00231439,0.00231488,0.00231535,0.00231583,0.00231629,0.00231675,0.0023172,0.00281914,0.00281973,0.00282031,0.00282089,0.00282146,0.00282203,0.00282258,0.00282313,0.00282368,0.00282421,0.00282474,0.00282527,0.00282578,0.0028263,0.0028268,0.0028273,0.00336506,0.0033657,0.00336633,0.00336695,0.00336757,0.00336818,0.00336878,0.00336938,0.00336997,0.00337055,0.00337113,0.0033717,0.00337227,0.00337283,0.00337339,0.00337394,0.0040151,0.00401581,0.00401651,0.0040172,0.00401789,0.00401857,0.00401925,0.00401992,0.00402059,0.00402125,0.0040219,0.00402255,0.00402319,0.00402383,0.00402447,0.0047729,0.00477372,0.00477453,0.00477534,0.00477614,0.00477693,0.00477772,0.00477851,0.00477929,0.00478006,0.00478083,0.0047816,0.00478236,0.00478311,0.00478386,0.00566649,0.00566747,0.00566845,0.00566942,0.00567038,0.00567134,0.0056723,0.00567325,0.0056742,0.00567514,0.00567608,0.00567701,0.00567794,0.00567887,0.00567979,0.00667879,0.00668,0.0066812,0.0066824,0.00668359,0.00668479,0.00668598,0.00668716,0.00668835,0.00668953,0.0066907,0.00669188,0.00669305,0.00669422,0.00669538,0.00790916,0.00791071,0.00791227,0.00791383,0.00791538,0.00791693,0.00791848,0.00792003,0.00792158,0.00792313,0.00792467,0.00792622,0.00792776,0.0079293,0.00931869,0.00932076,0.00932283,0.0093249,0.00932697,0.00932904,0.00933112,0.00933319,0.00933526,0.00933734,0.00933941,0.00934149,0.00934356,0.00934564,0.00934771,0.0109449,0.0109477,0.0109505,0.0109533,0.0109561,0.0109589,0.0109617,0.0109645,0.0109674,0.0109702,0.010973,0.0109759,0.0109787,0.0109815,0.0127948,0.0127986,0.0128025,0.0128063,0.0128101,0.012814,0.0128178,0.0128217,0.0128255,0.0128294,0.0128332,0.0128371,0.0128409,0.0128448,0.014834,0.0148391,0.0148442,0.0148494,0.0148545,0.0148596,0.0148648,0.0148699,0.014875,0.0148802,0.0148853,0.0148904,0.0148955,0.0149006,0.0169211,0.0169277,0.0169343,0.0169408,0.0169473,0.0169539,0.0169604,0.0169668,0.0169733,0.0169797,0.0169861,0.0169925,0.0169988,0.0188945,0.0189024,0.0189101,0.0189179,0.0189256,0.0189332,0.0189408,0.0189483,0.0189558,0.0189632,0.0189705,0.0189778,0.018985,0.0189922,0.0204932,0.0205014,0.0205095,0.0205175,0.0205254,0.0205332,0.0205409,0.0205485,0.020556,0.0205634,0.0205707,0.0205778,0.0205848,0.0214536,0.0214609,0.0214682,0.0214753,0.0214822,0.021489,0.0214956,0.021502,0.0215083,0.0215144,0.0215204,0.0215262,0.0215318,0.0216547,0.02166,0.0216651,0.02167,0.0216748,0.0216793,0.0216837,0.0216879,0.0216919,0.0216956,0.0216992,0.0217026,0.0217058,0.0210216,0.0210242,0.0210266,0.0210287,0.0210307,0.0210325,0.0210341,0.0210356,0.0210368,0.0210379,0.0210387,0.0210394,0.0210399,0.01977,0.0197701,0.0197699,0.0197696,0.0197692,0.0197685,0.0197678,0.0197669,0.0197658,0.0197646,0.0197632,0.0197616,0.01976,0.0181419,0.0181401,0.0181382,0.0181363,0.0181342,0.018132,0.0181297,0.0181272,0.0181247,0.0181221,0.0181193,0.0181165,0.0181135,0.0163403,0.0163377,0.0163349,0.0163322,0.0163293,0.0163264,0.0163234,0.0163203,0.0163172,0.016314,0.0163107,0.0163074,0.0145795,0.0145767,0.0145738,0.0145709,0.0145679,0.014565,0.0145619,0.0145589,0.0145558,0.0145526,0.0145494,0.0145462,0.0129548,0.0129522,0.0129495,0.0129468,0.0129441,0.0129414,0.0129386,0.0129358,0.012933,0.0129302,0.0129274,0.0129245,0.0129216,0.0115317,0.0115293,0.011527,0.0115246,0.0115222,0.0115198,0.0115174,0.011515,0.0115126,0.0115102,0.0115078,0.0115053,0.0102343,0.0102323,0.0102303,0.0102284,0.0102264,0.0102244,0.0102225,0.0102205,0.0102185,0.0102165,0.0102145,0.0102125,0.00918378,0.00918216,0.00918054,0.00917892,0.00917729,0.00917567,0.00917405,0.00917243,0.0091708,0.00916918,0.00916755,0.00826773,0.00826641,0.00826509,0.00826377,0.00826245,0.00826114,0.00825982,0.00825851,0.00825719,0.00825588,0.00825457,0.00825326,0.00748671,0.00748564,0.00748457,0.0074835,0.00748244,0.00748137,0.00748031,0.00747925,0.00747819,0.00747714,0.00747608,0.00747503,0.00681515,0.00681428,0.00681342,0.00681256,0.0068117,0.00681084,0.00680999,0.00680913,0.00680828,0.00680743,0.00680658,0.00625642,0.00625572,0.00625502,0.00625432,0.00625362,0.00625292,0.00625222,0.00625153,0.00625084,0.00625015,0.00624946,0.0058123,0.00581172,0.00581114,0.00581056,0.00580998,0.0058094,0.00580883,0.00580825,0.00580768,0.00580711,0.00580654,0.00580598,0.0053598,0.00535933,0.00535886,0.0053584,0.00535793,0.00535747,0.00535701,0.00535655,0.00535609,0.00535563,0.00535517,0.00502631,0.00502592,0.00502554,0.00502515,0.00502476,0.00502438,0.00502399,0.00502361,0.00502323,0.00502285,0.00502247,0.00471862,0.0047183,0.00471798,0.00471766,0.00471734,0.00471702,0.0047167,0.00471639,0.00471607,0.00471576,0.00471544,0.00446957,0.0044693,0.00446903,0.00446876,0.00446849,0.00446823,0.00446796,0.0044677,0.00446743,0.00446717,0.00446691,0.00424827,0.00424804,0.00424781,0.00424759,0.00424736,0.00424714,0.00424692,0.00424669,0.00424647,0.00424625,0.00405566,0.00405546,0.00405527,0.00405508,0.00405489,0.0040547,0.00405451,0.00405432,0.00405413,0.00405395,0.00405376,0.00385915,0.00385899,0.00385883,0.00385867,0.00385852,0.00385836,0.0038582,0.00385804,0.00385789,0.00385773,0.00375868,0.00375854,0.00375839,0.00375825,0.00375811,0.00375797,0.00375783,0.0037577,0.00375756,0.00375742,0.00375728,0.00358898,0.00358887,0.00358875,0.00358863,0.00358852,0.0035884,0.00358829,0.00358817,0.00358806,0.00358795,0.00351737,0.00351727,0.00351716,0.00351706,0.00351696,0.00351685,0.00351675,0.00351665,0.00351655,0.00351645,0.00351635,0.00341002,0.00340993,0.00340984,0.00340975,0.00340967,0.00340958,0.00340949,0.00340941,0.00340932,0.00340924,0.00333484,0.00333477,0.00333469,0.00333461,0.00333453,0.00333446,0.00333438,0.00333431,0.00333423,0.00333416,0.00325801,0.00325794,0.00325788,0.00325781,0.00325775,0.00325768,0.00325761,0.00325755,0.00325749,0.00325742,0.0032143,0.00321425,0.00321419,0.00321413,0.00321407,0.00321401,0.00321395,0.00321389,0.00321384,0.00321378,0.00313443,0.00313438,0.00313433,0.00313428,0.00313423,0.00313418,0.00313413,0.00313409,0.00313404,0.00313399,0.00308822,0.00308817,0.00308813,0.00308809,0.00308804,0.003088,0.00308796,0.00308792,0.00308788,0.00304092,0.00304088,0.00304084,0.00304081,0.00304077,0.00304073,0.0030407,0.00304066,0.00304063,0.00304059,0.00299246,0.00299243,0.0029924,0.00299237,0.00304364}};
	//
  int iqdata,iq,ichannel;
  double w1,w2,q;
  for(ichannel=0;ichannel<3;ichannel++){
    for(iq=0;iq<nqmax;iq++){
      q=qarray[iq];
      iqdata=int(floor(q)/delqdata);
      if(iqdata<=nqdata){
				w1=(delqdata*double(iqdata+1)-q)/delqdata;
				w2=1.0-w1;
				delta[ichannel][iq]=w1*data_delta[ichannel][iqdata]
				+w2*data_delta[ichannel][iqdata+1];
				ddeltadq[ichannel][iq]=w1*data_ddeltadq[ichannel][iqdata]
				+w2*data_ddeltadq[ichannel][iqdata+1];
      }
      else{
				delta[ichannel][iq]=0.0;
				ddeltadq[ichannel][iq]=0.0;
      }
    }
  }
	
}

#endif
